@@ -6,13 +6,13 @@ Clean chat context focused on conversation state and tool coordination.
 from __future__ import annotations
 
 import logging
-from typing import Any, Dict, List, AsyncIterator
+from typing import Any, Dict, List, AsyncIterator, Optional
 
-from rich import print
-from rich.console import Console
+from chuk_term.ui import output
 
 from mcp_cli.chat.system_prompt import generate_system_prompt
 from mcp_cli.tools.manager import ToolManager
+from mcp_cli.tools.models import ToolInfo, ServerInfo
 from mcp_cli.model_manager import ModelManager
 
 logger = logging.getLogger(__name__)
@@ -21,12 +21,12 @@ logger = logging.getLogger(__name__)
 class ChatContext:
     """
     Chat context focused on conversation state and tool coordination.
-    
+
     Responsibilities:
     - Conversation history management
     - Tool discovery and adaptation coordination
     - Session state (exit requests, etc.)
-    
+
     Model management is completely delegated to ModelManager.
     """
 
@@ -40,49 +40,54 @@ class ChatContext:
         """
         self.tool_manager = tool_manager
         self.model_manager = model_manager
-        
+
         # Conversation state
         self.exit_requested = False
         self.conversation_history: List[Dict[str, Any]] = []
-        
+        self.tool_history: List[Dict[str, Any]] = []  # Track tool execution history
+
         # Tool state (filled during initialization)
-        self.tools: List[Dict[str, Any]] = []
-        self.internal_tools: List[Dict[str, Any]] = []
-        self.server_info: List[Dict[str, Any]] = []
+        self.tools: List[ToolInfo] = []
+        self.internal_tools: List[ToolInfo] = []
+        self.server_info: List[ServerInfo] = []
         self.tool_to_server_map: Dict[str, str] = {}
-        self.openai_tools: List[Dict[str, Any]] = []
+        self.openai_tools: List[
+            Dict[str, Any]
+        ] = []  # These remain dicts for OpenAI API
         self.tool_name_mapping: Dict[str, str] = {}
-        
+
         logger.debug(f"ChatContext created with {self.provider}/{self.model}")
 
     @classmethod
     def create(
         cls,
         tool_manager: ToolManager,
-        provider: str = None,
-        model: str = None,
-        api_base: str = None,
-        api_key: str = None,
-    ) -> 'ChatContext':
+        provider: str | None = None,
+        model: str | None = None,
+        api_base: str | None = None,
+        api_key: str | None = None,
+    ) -> "ChatContext":
         """
         Factory method for convenient creation.
-        
+
         Args:
             tool_manager: Tool management interface
             provider: Provider to switch to (optional)
             model: Model to switch to (optional)
             api_base: API base URL override (optional)
             api_key: API key override (optional)
-            
+
         Returns:
             Configured ChatContext instance
         """
         model_manager = ModelManager()
-        
+
         # Configure provider if API settings provided
         if provider and (api_base or api_key):
-            model_manager.configure_provider(provider, api_key=api_key, api_base=api_base)
-        
+            model_manager.configure_provider(
+                provider, api_key=api_key, api_base=api_base
+            )
+
         # Switch model if requested
         if provider and model:
             model_manager.switch_model(provider, model)
@@ -90,7 +95,7 @@ class ChatContext:
             model_manager.switch_provider(provider)
         elif model:
             model_manager.switch_to_model(model)
-        
+
         return cls(tool_manager, model_manager)
 
     # ── Properties that delegate to ModelManager ──────────────────────────
@@ -112,84 +117,120 @@ class ChatContext:
     # ── Initialization ────────────────────────────────────────────────────
     async def initialize(self) -> bool:
         """Initialize tools and conversation state."""
-        console = Console()
         try:
-            with console.status("[bold cyan]Setting up chat environment…[/bold cyan]", spinner="dots"):
-                await self._initialize_tools()
-                self._initialize_conversation()
-            
-            if not self.tools:
-                print("[yellow]No tools available. Chat functionality may be limited.[/yellow]")
+            await self._initialize_tools()
+            self._initialize_conversation()
 
-            logger.info(f"ChatContext ready: {len(self.tools)} tools, {self.provider}/{self.model}")
+            if not self.tools:
+                output.print(
+                    "[yellow]No tools available. Chat functionality may be limited.[/yellow]"
+                )
+
+            logger.info(
+                f"ChatContext ready: {len(self.tools)} tools, {self.provider}/{self.model}"
+            )
             return True
-            
+
         except Exception as exc:
             logger.exception("Error initializing chat context")
-            print(f"[red]Error initializing chat context: {exc}[/red]")
+            output.print(f"[red]Error initializing chat context: {exc}[/red]")
             return False
 
     async def _initialize_tools(self) -> None:
         """Initialize tool discovery and adaptation."""
-        # Get tools from ToolManager
-        tool_infos = await self.tool_manager.get_unique_tools()
-        
-        self.tools = [
-            {
-                "name": t.name,
-                "description": t.description,
-                "parameters": t.parameters,
-                "namespace": t.namespace,
-                "supports_streaming": getattr(t, "supports_streaming", False),
-            }
-            for t in tool_infos
-        ]
-        
-        # Get server info
-        raw_infos = await self.tool_manager.get_server_info()
-        self.server_info = [
-            {"id": s.id, "name": s.name, "tools": s.tool_count, "status": s.status}
-            for s in raw_infos
-        ]
-        
-        # Build tool-to-server mapping
-        self.tool_to_server_map = {t["name"]: t["namespace"] for t in self.tools}
-        
+        # Get tools from ToolManager - already returns ToolInfo objects
+        self.tools = await self.tool_manager.get_unique_tools()
+
+        # Get server info - already returns ServerInfo objects
+        self.server_info = await self.tool_manager.get_server_info()
+
+        # Build tool-to-server mapping using ToolInfo objects
+        self.tool_to_server_map = {t.name: t.namespace for t in self.tools}
+
         # Adapt tools for current provider
         await self._adapt_tools_for_provider()
-        
+
         # Keep copy for system prompt
         self.internal_tools = list(self.tools)
 
+    def find_tool_by_name(self, name: str) -> Optional[ToolInfo]:
+        """Find a tool by its name (handles both simple and namespaced names)."""
+        # First try exact match
+        for tool in self.tools:
+            if tool.name == name or tool.fully_qualified_name == name:
+                return tool
+
+        # Try partial match (just the tool name without namespace)
+        simple_name = name.split(".")[-1] if "." in name else name
+        for tool in self.tools:
+            if tool.name == simple_name:
+                return tool
+
+        return None
+
+    def find_server_by_name(self, name: str) -> Optional[ServerInfo]:
+        """Find a server by its name."""
+        for server in self.server_info:
+            if server.name == name or server.namespace == name:
+                return server
+        return None
+
     async def _adapt_tools_for_provider(self) -> None:
         """Adapt tools for current provider."""
         try:
             if hasattr(self.tool_manager, "get_adapted_tools_for_llm"):
-                tools_and_mapping = await self.tool_manager.get_adapted_tools_for_llm(self.provider)
+                tools_and_mapping = await self.tool_manager.get_adapted_tools_for_llm(
+                    self.provider
+                )
                 self.openai_tools = tools_and_mapping[0]
                 self.tool_name_mapping = tools_and_mapping[1]
-                logger.debug(f"Adapted {len(self.openai_tools)} tools for {self.provider}")
+                logger.debug(
+                    f"Adapted {len(self.openai_tools)} tools for {self.provider}"
+                )
             else:
                 # Fallback to generic tools
                 self.openai_tools = await self.tool_manager.get_tools_for_llm()
                 self.tool_name_mapping = {}
         except Exception as exc:
             logger.warning(f"Error adapting tools: {exc}")
-            # Final fallback - use basic conversion
-            from mcp_cli.tools.manager import ToolManager
-            self.openai_tools = ToolManager.convert_to_openai_tools(self.tools)
+            # Final fallback - use the raw tool format
+            self.openai_tools = await self.tool_manager.get_tools_for_llm()
             self.tool_name_mapping = {}
 
     def _initialize_conversation(self) -> None:
         """Initialize conversation with system prompt."""
-        system_prompt = generate_system_prompt(self.internal_tools)
+        # Convert ToolInfo objects to dicts for system prompt generation
+        tools_for_prompt = []
+        for tool in self.internal_tools:
+            if hasattr(tool, "to_openai_format"):
+                tools_for_prompt.append(tool.to_openai_format())
+            elif isinstance(tool, dict):
+                tools_for_prompt.append(tool)
+            else:
+                # Fallback - convert to dict
+                tools_for_prompt.append(
+                    {
+                        "type": "function",
+                        "function": {
+                            "name": tool.name if hasattr(tool, "name") else str(tool),
+                            "description": tool.description
+                            if hasattr(tool, "description")
+                            else "No description",
+                            "parameters": tool.parameters
+                            if hasattr(tool, "parameters")
+                            else {},
+                        },
+                    }
+                )
+
+        system_prompt = generate_system_prompt(tools_for_prompt)
         self.conversation_history = [{"role": "system", "content": system_prompt}]
 
     # ── Model change handling ─────────────────────────────────────────────
     async def refresh_after_model_change(self) -> None:
         """
         Refresh context after ModelManager changes the model.
-        
+
         Call this after model_manager.switch_model() to update tools.
         ModelManager handles client refresh automatically.
         """
@@ -200,8 +241,10 @@ class ChatContext:
     async def execute_tool(self, tool_name: str, arguments: Dict[str, Any]) -> Any:
         """Execute a tool."""
         return await self.tool_manager.execute_tool(tool_name, arguments)
-    
-    async def stream_execute_tool(self, tool_name: str, arguments: Dict[str, Any]) -> AsyncIterator[Any]:
+
+    async def stream_execute_tool(
+        self, tool_name: str, arguments: Dict[str, Any]
+    ) -> AsyncIterator[Any]:
         """Execute a tool with streaming."""
         async for result in self.tool_manager.stream_execute_tool(tool_name, arguments):
             yield result
@@ -225,7 +268,11 @@ class ChatContext:
 
     def clear_conversation_history(self, keep_system_prompt: bool = True) -> None:
         """Clear conversation history."""
-        if keep_system_prompt and self.conversation_history and self.conversation_history[0].get("role") == "system":
+        if (
+            keep_system_prompt
+            and self.conversation_history
+            and self.conversation_history[0].get("role") == "system"
+        ):
             system_prompt = self.conversation_history[0]
             self.conversation_history = [system_prompt]
         else:
@@ -234,10 +281,15 @@ class ChatContext:
     def regenerate_system_prompt(self) -> None:
         """Regenerate system prompt with current tools."""
         system_prompt = generate_system_prompt(self.internal_tools)
-        if self.conversation_history and self.conversation_history[0].get("role") == "system":
+        if (
+            self.conversation_history
+            and self.conversation_history[0].get("role") == "system"
+        ):
             self.conversation_history[0]["content"] = system_prompt
         else:
-            self.conversation_history.insert(0, {"role": "system", "content": system_prompt})
+            self.conversation_history.insert(
+                0, {"role": "system", "content": system_prompt}
+            )
 
     # ── Simple getters ────────────────────────────────────────────────────
     def get_tool_count(self) -> int:
@@ -285,8 +337,14 @@ class ChatContext:
             self.model_manager = context_dict["model_manager"]
 
         # Tool state updates (for command handlers that modify tools)
-        for key in ["tools", "internal_tools", "server_info", "tool_to_server_map", 
-                   "openai_tools", "tool_name_mapping"]:
+        for key in [
+            "tools",
+            "internal_tools",
+            "server_info",
+            "tool_to_server_map",
+            "openai_tools",
+            "tool_name_mapping",
+        ]:
             if key in context_dict:
                 setattr(self, key, context_dict[key])
 
@@ -315,8 +373,10 @@ class ChatContext:
         }
 
     def __repr__(self) -> str:
-        return (f"ChatContext(provider='{self.provider}', model='{self.model}', "
-                f"tools={len(self.tools)}, messages={self.get_conversation_length()})")
+        return (
+            f"ChatContext(provider='{self.provider}', model='{self.model}', "
+            f"tools={len(self.tools)}, messages={self.get_conversation_length()})"
+        )
 
     def __str__(self) -> str:
         return f"Chat session with {self.provider}/{self.model} ({len(self.tools)} tools, {self.get_conversation_length()} messages)"
@@ -326,24 +386,25 @@ class ChatContext:
 # For testing - separate class to keep main ChatContext clean
 # ═══════════════════════════════════════════════════════════════════════════════════
 
+
 class TestChatContext(ChatContext):
     """
     Test-specific ChatContext that works with stream_manager instead of ToolManager.
-    
+
     Separated from main ChatContext to keep it clean.
     """
-    
+
     def __init__(self, stream_manager: Any, model_manager: ModelManager):
         """Create test context with stream_manager."""
         # Initialize base attributes without calling super().__init__
-        self.tool_manager = None  # Tests don't use ToolManager
+        self.tool_manager = None  # type: ignore[assignment]  # Tests don't use ToolManager
         self.stream_manager = stream_manager
         self.model_manager = model_manager
-        
+
         # Conversation state
         self.exit_requested = False
         self.conversation_history = []
-        
+
         # Tool state
         self.tools = []
         self.internal_tools = []
@@ -351,26 +412,26 @@ class TestChatContext(ChatContext):
         self.tool_to_server_map = {}
         self.openai_tools = []
         self.tool_name_mapping = {}
-        
+
         logger.debug(f"TestChatContext created with {self.provider}/{self.model}")
 
     @classmethod
     def create_for_testing(
         cls,
         stream_manager: Any,
-        provider: str = None,
-        model: str = None,
-    ) -> 'TestChatContext':
+        provider: str | None = None,
+        model: str | None = None,
+    ) -> "TestChatContext":
         """Factory for test contexts."""
         model_manager = ModelManager()
-        
+
         if provider and model:
             model_manager.switch_model(provider, model)
         elif provider:
             model_manager.switch_provider(provider)
         elif model:
             model_manager.switch_to_model(model)
-        
+
         return cls(stream_manager, model_manager)
 
     async def _initialize_tools(self) -> None:
@@ -383,18 +444,35 @@ class TestChatContext(ChatContext):
 
         # Get server info
         self.server_info = list(self.stream_manager.get_server_info())
-        
+
         # Build mappings
         self.tool_to_server_map = {
-            t["name"]: self.stream_manager.get_server_for_tool(t["name"])
+            (
+                t["name"] if isinstance(t, dict) else t.name
+            ): self.stream_manager.get_server_for_tool(
+                t["name"] if isinstance(t, dict) else t.name
+            )
             for t in self.tools
         }
-        
-        # Use basic tool conversion for tests
-        from mcp_cli.tools.manager import ToolManager
-        self.openai_tools = ToolManager.convert_to_openai_tools(self.tools)
+
+        # Convert tools to OpenAI format for tests
+        self.openai_tools = [
+            {
+                "type": "function",
+                "function": {
+                    "name": t.get("name", "unknown") if isinstance(t, dict) else t.name,
+                    "description": t.get("description", "")
+                    if isinstance(t, dict)
+                    else t.description,
+                    "parameters": t.get("parameters", {})
+                    if isinstance(t, dict)
+                    else t.parameters,
+                },
+            }
+            for t in self.tools
+        ]
         self.tool_name_mapping = {}
-        
+
         # Copy for system prompt
         self.internal_tools = list(self.tools)
 
@@ -407,4 +485,4 @@ class TestChatContext(ChatContext):
 
     async def get_server_for_tool(self, tool_name: str) -> str:
         """Get server for tool from stream_manager."""
-        return self.stream_manager.get_server_for_tool(tool_name) or "Unknown"
\ No newline at end of file
+        return self.stream_manager.get_server_for_tool(tool_name) or "Unknown"
