@@ -7,6 +7,7 @@ Tiny helper for “run an async coroutine from possibly-sync code”.
 * If called **inside** a running loop → we raise, so callers know to
   switch to the `*_async` variant instead of silently returning junk.
 """
+
 from __future__ import annotations
 
 import asyncio
@@ -19,7 +20,17 @@ def run_blocking(coro: Awaitable[T]) -> T:
     try:
         loop = asyncio.get_running_loop()
     except RuntimeError:  # totally sync context
-        return asyncio.run(coro)
+        # asyncio.run expects a coroutine, not just any awaitable
+        if asyncio.iscoroutine(coro):
+            coro_result: T = asyncio.run(coro)
+            return coro_result
+        else:
+            # If it's not a coroutine, wrap it
+            async def _wrapper() -> T:
+                return await coro
+
+            wrapped_result: T = asyncio.run(_wrapper())
+            return wrapped_result
 
     if loop.is_running():
         raise RuntimeError(
